
classdef MokuArbitraryWaveGen < moku
    % MOKUARBITRARYWAVEGEN(IpAddr) 
    % Driver class for the Moku:ArbitraryWaveGen
    %
    % MOKUARBITRARYWAVEGEN Properties:
    %   IP - IP address of the currently connected Moku:Lab device.
    %   Instrument - Name of the currently running instrument.
    %   Timeout - Command timeout period (sec)
    %
    % MOKUARBITRARYWAVEGEN Methods:
    %   commit  - Apply all modified settings.
    %   enable_output  - Enable or disable the ArbitraryWaveGen output(s).
    %   gen_off  - DEPRECATED Turn ArbitraryWaveGen output(s) off.
    %   gen_waveform  - Configure and enable the Arbitrary Waveform on the given output channel.
    %   get_data  - Get full-resolution data from the instrument.
    %   get_frequency  - Returns the frequency of the output waveform on the selected channel.
    %   get_frontend  - Get the analog frontend configuration.
    %   get_realtime_data  - Get downsampled data from the instrument with low latency.
    %   get_samplerate  - :return: The current instrument sample rate (Hz) 
    %   reset_phase  - resets the channels phase accumulator to zero
    %   set_defaults  - Sets the Arbitrary Waveform Generator instrument to sane defaults
    %   set_framerate  - Set framerate 
    %   set_frontend  - Configures gain, coupling and termination for each channel.
    %   set_precision_mode  - Change aquisition mode between downsampling and decimation.
    %   set_samplerate  - Manually set the sample rate of the instrument.
    %   set_timebase  - Set the left- and right-hand span for the time axis.
    %   set_waveform_trigger  - Specify what constitutes a trigger event for the given output channel. This takes effect
    %   set_waveform_trigger_output  - Enables triggered output mode on the specified channel and configures 'how' to output the 
    %   set_xmode  - Set rendering mode for the horizontal axis.
    %   sync_phase  - Resets the phase accumulator of both output waveforms.
    %   write_lut  - Writes the signal lookup table to memory in the Moku:Lab.
    %
    % See also moku

    methods
        function obj = MokuArbitraryWaveGen(IpAddr)
            obj@moku(IpAddr, 'arbitrarywavegen');
        end

        function commit(obj)
            % COMMIT()
            % Apply all modified settings.
            % 
            % .. note::
            % 
            %     If the `autocommit` feature has been turned off, this function can be used to manually apply any instrument
            %     settings to the Moku device. These instrument settings are those configured by calling all *set_* and *gen_* type
            %     functions. Manually calling this function allows you to atomically apply many instrument settings at once.
            params = struct();
            mokuctl(obj, 'commit', params);
        end

        function enable_output(obj, varargin)
            % ENABLE_OUTPUT()
            % ENABLE_OUTPUT(...,param1,val1,param2,val2,...)
            % Enable or disable the ArbitraryWaveGen output(s).
            % 
            % If *ch* is None (the default), both channels will be acted upon,
            % otherwise just the one specified by the argument.
            % 
            % :type ch: int; {1,2} or None
            % :param ch: Output channel, or both.
            % 
            % :type en: bool
            % :param en: Enable the specified output channel(s).
            % 
            % :raises ValueError: Invalid parameters
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'enable_output', params);
        end

        function gen_off(obj, varargin)
            % GEN_OFF()
            % GEN_OFF(...,param1,val1,param2,val2,...)
            % DEPRECATED Turn ArbitraryWaveGen output(s) off.
            % 
            % The channel will be turned on when configuring the waveform type but can be turned off
            % using this function. If *ch* is None (the default), both channels will be turned off,
            % otherwise just the one specified by the argument.
            % 
            % :type ch: int; {1,2} or None
            % :param ch: Channel to turn off, or both.
            % 
            % :raises ValueError: Invalid parameters
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'gen_off', params);
        end

        function gen_waveform(obj, ch, period, amplitude, varargin)
            % GEN_WAVEFORM(ch, period, amplitude)
            % GEN_WAVEFORM(...,param1,val1,param2,val2,...)
            % Configure and enable the Arbitrary Waveform on the given output channel.
            % 
            % The look-up table for this channel's output waveform should have been loaded beforehand using :any:`write_lut`.
            % 
            % The Arbitrary Waveform Generator has the ability to insert a deadtime between cycles of the look-up
            % table. This time is specified in cycles of the waveform. During this time, the output will be held
            % at the given *dead_voltage*.  This allows the user to, for example, generate infrequent pulses without
            % using space in the LUT to specify the time between, keeping the full LUT size to provide a high-resolution
            % pulse shape.
            % 
            % Where the period and look-up table contents are set such that there isn't exactly one LUT point per output
            % sample, the AWG instrument can optionally provide a linear interpolation between LUT points.
            % 
            % This function enables the output channel by default. If you wish to enable the outputs simultaneously, you
            % should set the `en` parameter to False and enable both when desired using :any:`enable_output`.
            % 
            % :type ch: int; {1,2}
            % :param ch: Channel on which to generate the wave
            % 
            % :type period: float, [4e-9, 1];
            % :param period: period of the signal in seconds
            % 
            % :type amplitude: float, [0.0,2.0] Vpp
            % :param amplitude: Waveform peak-to-peak amplitude
            % 
            % :type phase: float, [0-360] degrees
            % :param phase: Phase offset of the wave
            % 
            % :type offset: float, [-1.0,1.0] Volts
            % :param offset: DC offset applied to the waveform
            % 
            % :type interpolation: bool [True, False]
            % :param interpolation: Enable linear interpolation of LUT entries
            % 
            % :type dead_time: float [0, 2e18] cyc
            % :param dead_time: number of cycles which show the dead voltage. Use 0 for no dead time
            % 
            % :type dead_voltage: float [-2.0,2.0] V
            % :param dead_voltage: signal level during dead time in Volts
            % 
            % :type en: bool
            % :param en: Enable output
            % 
            % :raises ValueError: if the parameters  is invalid
            % :raises ValueOutOfRangeException: if wave parameters are out of range
            % :raises InvalidParameterException: if the parameters are the wrong types
            params = struct('ch',ch,'period',period,'amplitude',amplitude);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'gen_waveform', params);
        end

        function ret = get_data(obj, varargin)
            % GET_DATA()
            % GET_DATA(...,param1,val1,param2,val2,...)
            % Get full-resolution data from the instrument.
            % 
            % This will pause the instrument and download the entire contents of the instrument's
            % internal memory. This may include slightly more data than the instrument is set up
            % to record due to rounding of some parameters in the instrument.
            % 
            % All settings must be committed before you call this function. If *pymoku.autocommit=True*
            % (the default) then this will always be true, otherwise you will need to have called
            % :any:`commit` first.
            % 
            % The download process may take a second or so to complete. If you require high rate
            % data, e.g. for rendering a plot, see `get_realtime_data`.
            % 
            % If the *wait* parameter is true (the default), this function will wait for any new
            % settings to be applied before returning. That is, if you have set a new timebase (for example),
            % calling this with *wait=True* will guarantee that the data returned has this new timebase.
            % 
            % Note that if instrument configuration is changed, a trigger event must occur before data
            % captured with that configuration set can become available. This can take an arbitrary amount
            % of time. For this reason the *timeout* should be set appropriately.
            % 
            % :type timeout: float
            % :param timeout: Maximum time to wait for new data, or *None* for indefinite.
            % 
            % :type wait: bool
            % :param wait: If *true* (default), waits for a new waveform to be captured with the most
            %         recently-applied settings, otherwise just return the most recently captured valid data.
            % 
            % :return: :any:`InstrumentData` subclass, specific to the instrument.
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'get_data', params);
        end

        function ret = get_frequency(obj, ch)
            % GET_FREQUENCY(ch)
            % Returns the frequency of the output waveform on the selected channel.
            % 
            % :type ch: int; {1,2}
            % :param ch: Output channel
            % 
            % :raises ValueError: if the channel number is invalid
            params = struct('ch',ch);
            ret = ...
            mokuctl(obj, 'get_frequency', params);
        end

        function ret = get_frontend(obj, channel)
            % GET_FRONTEND(channel)
            % Get the analog frontend configuration.
            % 
            % :type channel: int; {1,2}
            % :param channel: Channel for which the relay settings are being retrieved
            % 
            % :return: Array of bool with the front end configuration of channels
            %         - [0] 50 Ohm
            %         - [1] 10xAttenuation
            %         - [2] AC Coupling
            params = struct('channel',channel);
            ret = ...
            mokuctl(obj, 'get_frontend', params);
        end

        function ret = get_realtime_data(obj, varargin)
            % GET_REALTIME_DATA()
            % GET_REALTIME_DATA(...,param1,val1,param2,val2,...)
            % Get downsampled data from the instrument with low latency.
            % 
            % Returns a new :any:`InstrumentData` subclass (instrument-specific), containing
            % a version of the data that may have been downsampled from the original in order to
            % be transferred quickly.
            % 
            % This function always returns a new object at `framerate` (10Hz by default), whether
            % or not there is new data in that object. This can be verified by checking the return
            % object's *waveformid* parameter, which increments each time a new waveform is captured
            % internally.
            % 
            % The downsampled, low-latency nature of this data makes it particularly suitable for
            % plotting in real time. If you require high-accuracy, high-resolution data for analysis,
            % see `get_data`.
            % 
            % If the *wait* parameter is true (the default), this function will wait for any new
            % settings to be applied before returning. That is, if you have set a new timebase (for example),
            % calling this with *wait=True* will guarantee that the data returned has this new timebase.
            % 
            % Note that if instrument configuration is changed, a trigger event must occur before data
            % captured with that configuration set can become available. This can take an arbitrary amount
            % of time. For this reason the *timeout* should be set appropriately.
            % 
            % :type timeout: float
            % :param timeout: Maximum time to wait for new data, or *None* for indefinite.
            % 
            % :type wait: bool
            % :param wait: If *true* (default), waits for a new waveform to be captured with the most
            %         recently-applied settings, otherwise just return the most recently captured valid data.
            % 
            % :return: :any:`InstrumentData` subclass, specific to the instrument.
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'get_realtime_data', params);
        end

        function ret = get_samplerate(obj)
            % GET_SAMPLERATE()
            % :return: The current instrument sample rate (Hz) 
            params = struct();
            ret = ...
            mokuctl(obj, 'get_samplerate', params);
        end

        function reset_phase(obj, ch)
            % RESET_PHASE(ch)
            % resets the channels phase accumulator to zero
            % 
            % :type ch: int; {1,2}
            % :param ch: Channel on which the reset is performed
            % 
            % :raises ValueError: if the channel number is invalid
            params = struct('ch',ch);
            mokuctl(obj, 'reset_phase', params);
        end

        function set_defaults(obj)
            % SET_DEFAULTS()
            % Sets the Arbitrary Waveform Generator instrument to sane defaults
            %                 
            params = struct();
            mokuctl(obj, 'set_defaults', params);
        end

        function set_framerate(obj, fr)
            % SET_FRAMERATE(fr)
            % Set framerate 
            params = struct('fr',fr);
            mokuctl(obj, 'set_framerate', params);
        end

        function ret = set_frontend(obj, channel, varargin)
            % SET_FRONTEND(channel)
            % SET_FRONTEND(...,param1,val1,param2,val2,...)
            % Configures gain, coupling and termination for each channel.
            % 
            % :type channel: int; {1,2}
            % :param channel: Channel to which the settings should be applied
            % 
            % :type fiftyr: bool
            % :param fiftyr: 50Ohm termination; default is 1MOhm.
            % 
            % :type atten: bool
            % :param atten: Turn on 10x attenuation. Changes the dynamic range between 1Vpp and 10Vpp.
            % 
            % :type ac: bool
            % :param ac: AC-couple; default DC.
            params = struct('channel',channel);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'set_frontend', params);
        end

        function set_precision_mode(obj, state)
            % SET_PRECISION_MODE(state)
            % Change aquisition mode between downsampling and decimation.
            % Precision mode, a.k.a Decimation, samples at full rate and applies a low-pass filter to the data. This improves
            % precision. Normal mode works by direct downsampling, throwing away points it doesn't need.
            % 
            % :param state: Select Precision Mode
            % :type state: bool
            params = struct('state',state);
            mokuctl(obj, 'set_precision_mode', params);
        end

        function set_samplerate(obj, samplerate, varargin)
            % SET_SAMPLERATE(samplerate)
            % SET_SAMPLERATE(...,param1,val1,param2,val2,...)
            % Manually set the sample rate of the instrument.
            % 
            % The sample rate is automatically calculated and set in :any:`set_timebase`.
            % 
            % This interface allows you to specify the rate at which data is sampled, and set
            % a trigger offset in number of samples. This interface is useful for datalogging and capturing
            % of data frames.
            % 
            % :type samplerate: float; *0 < samplerate <= MAX_SAMPLERATE smp/s*
            % :param samplerate: Target samples per second. Will get rounded to the nearest allowable unit.
            % 
            % :type trigger_offset: int; *-2^16 < trigger_offset < 2^31*
            % :param trigger_offset: Number of samples before (-) or after (+) the trigger point to start capturing.
            % 
            % :raises ValueOutOfRangeException: if either parameter is out of range.
            params = struct('samplerate',samplerate);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_samplerate', params);
        end

        function set_timebase(obj, t1, t2)
            % SET_TIMEBASE(t1, t2)
            % Set the left- and right-hand span for the time axis.
            % Units are seconds relative to the trigger point.
            % 
            % :type t1: float
            % :param t1:
            %         Time, in seconds, from the trigger point to the left of screen. This may be negative (trigger on-screen)
            %         or positive (trigger off the left of screen).
            % 
            % :type t2: float
            % :param t2: As *t1* but to the right of screen.
            % 
            % :raises InvalidConfigurationException: if the timebase is backwards or zero.
            params = struct('t1',t1,'t2',t2);
            mokuctl(obj, 'set_timebase', params);
        end

        function set_waveform_trigger(obj, ch, source, edge, level, varargin)
            % SET_WAVEFORM_TRIGGER(ch, source, edge, level)
            % SET_WAVEFORM_TRIGGER(...,param1,val1,param2,val2,...)
            % Specify what constitutes a trigger event for the given output channel. This takes effect
            %         only when the channel has triggered output mode enabled (see :any:`set_waveform_trigger_mode').
            % 
            % :type ch: int; {1,2}
            % :param ch: Output channel to set triggering on
            % 
            % :type source: string, {'in1','in2','ext'}
            % :param source: Trigger source. May be either input channel, or the external 'Trig' back-panel connector 
            %         allowing triggering from an externally-generated digital [LV]TTL or CMOS signal.
            % 
            % :type edge: string, {'rising','falling','both'}
            % :param edge: Which edge to trigger on. In 'Pulse Width' mode this specifies whether the pulse is positive (rising)
            %                 or negative (falling), with the 'both' option being invalid.
            % 
            % :type level: float, [-5.0, 5.0] volts
            % :param level: Trigger level. Ignored in 'ext' mode.
            % 
            % :type minwidth: float, seconds
            % :param minwidth: Minimum Pulse Width. 0 <= minwidth < (2^32/samplerate). Can't be used with maxwidth.
            % 
            % :type maxwidth: float, seconds
            % :param maxwidth: Maximum Pulse Width. 0 <= maxwidth < (2^32/samplerate). Can't be used with minwidth.
            % 
            % :type hysteresis: bool
            % :param hysteresis: Enable hysteresis around trigger point.
            params = struct('ch',ch,'source',source,'edge',edge,'level',level);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_waveform_trigger', params);
        end

        function set_waveform_trigger_output(obj, ch, varargin)
            % SET_WAVEFORM_TRIGGER_OUTPUT(ch)
            % SET_WAVEFORM_TRIGGER_OUTPUT(...,param1,val1,param2,val2,...)
            % Enables triggered output mode on the specified channel and configures 'how' to output the 
            %         set waveform on a trigger event.
            % 
            % :type ch: int; {1,2}
            % :param ch: Output channel to configure
            % 
            % :type trig_en: bool;
            % :param trig_en: Enables triggering mode on the specified output channel
            % 
            % :type single: bool;
            % :param single; Enables single mode. Outputs a single waveform (vs continuous) per trigger event. 
            % 
            % :type duration: float; [0.0, 1e11] seconds
            % :param duration: Total time that the triggered output should be generated (leave 0 for continuous). 
            %         Note the duration resolution is 8ns.
            % 
            % :type hold_last: bool
            % :param hold_last: Hold the last value of the waveform for the duration of the triggered output.
            params = struct('ch',ch);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_waveform_trigger_output', params);
        end

        function set_xmode(obj, xmode)
            % SET_XMODE(xmode)
            % Set rendering mode for the horizontal axis.
            % 
            % :type xmode: string, {'roll','sweep','fullframe'}
            % :param xmode:
            %         Respectively; Roll Mode (scrolling), Sweep Mode (normal oscilloscope trace sweeping across the screen)
            %         or Full Frame (like sweep, but waits for the frame to be completed).
            params = struct('xmode',xmode);
            mokuctl(obj, 'set_xmode', params);
        end

        function sync_phase(obj)
            % SYNC_PHASE()
            % Resets the phase accumulator of both output waveforms.
            %                 
            params = struct();
            mokuctl(obj, 'sync_phase', params);
        end

        function write_lut(obj, ch, data, varargin)
            % WRITE_LUT(ch, data)
            % WRITE_LUT(...,param1,val1,param2,val2,...)
            % Writes the signal lookup table to memory in the Moku:Lab.
            % 
            % You can also choose the output rate of the AWG, which influences the
            % maximum length of the look-up table as follows:
            % 
            % - 1000MSPS: 8192 points per channel
            % - 500MSPS: 16384 points per channel
            % - 250MSPS: 32768 points per channel
            % - 125MSPS: 65536 points per channel
            % 
            % If you don't specify a mode, the fastest output rate for the given data
            % length will be automatically chosen. This is correct in almost all
            % circumstances.
            % 
            % If you specify a particular mode along with a data array too big for
            % that mode, the behaviour is undefined.
            % 
            % To avoid unexpected output signals during write, disable the outputs
            % by using the :any:`enable_output` function.
            % 
            % :type ch: int; {1,2}
            % :param ch: Output channel to load the LUT to
            % 
            % :type data: float array;
            % :param data: Lookup table coefficients normalised to range [-1.0, 1.0].
            % 
            % :type mode: int; {125, 250, 500, 1000} MSmps
            % :param mode: defines the output sample rate of the AWG.
            % 
            % :raises ValueError: if the channel is invalid
            % :raises ValueOutOfRangeException: if wave parameters are out of range
            params = struct('ch',ch,'data',data);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'write_lut', params);
        end

    end
end
