
classdef MokuWaveformGenerator < moku
    % MOKUWAVEFORMGENERATOR(IpAddr) 
    % Driver class for the Moku:WaveformGenerator
    %
    % MOKUWAVEFORMGENERATOR Properties:
    %   IP - IP address of the currently connected Moku:Lab device.
    %   Instrument - Name of the currently running instrument.
    %   Timeout - Command timeout period (sec)
    %
    % MOKUWAVEFORMGENERATOR Methods:
    %   commit  - Apply all modified settings.
    %   gen_modulate  - Set up modulation on an output channel.
    %   gen_modulate_off  - Turn off modulation for the specified output channel.
    %   gen_off  - Turn Waveform Generator output(s) off.
    %   gen_rampwave  - Generate a Ramp with the given parameters on the given channel.
    %   gen_sinewave  - Generate a Sine Wave with the given parameters on the given channel.
    %   gen_squarewave  - Generate a Square Wave with given parameters on the given channel.
    %   gen_trigger_off  - Turn off trigger/sweep mode for the specified output channel.
    %   set_defaults  - Can be extended in implementations to set initial state 
    %   set_trigger  - Configure gated, start, ncycle or sweep trigger mode on target channel.
    %
    % See also moku

    methods
        function obj = MokuWaveformGenerator(IpAddr)
            obj@moku(IpAddr, 'waveformgenerator');
        end

        function commit(obj)
            % COMMIT()
            % Apply all modified settings.
            % 
            % .. note::
            % 
            %     If the `autocommit` feature has been turned off, this function can be used to manually apply any instrument
            %     settings to the Moku device. These instrument settings are those configured by calling all *set_* and *gen_* type
            %     functions. Manually calling this function allows you to atomically apply many instrument settings at once.
            params = struct();
            mokuctl(obj, 'commit', params);
        end

        function gen_modulate(obj, ch, mtype, source, depth, varargin)
            % GEN_MODULATE(ch, mtype, source, depth)
            % GEN_MODULATE(...,param1,val1,param2,val2,...)
            % Set up modulation on an output channel.
            % 
            % :type ch: int; {1,2}
            % :param ch: Channel to modulate
            % 
            % :type mtype: string, {amplitude', 'frequency', 'phase'}
            % :param mtype:  Modulation type. Respectively Off, Amplitude, Frequency and Phase modulation.
            % 
            % :type source: string, {'internal', 'in', 'out'}
            % :param source: Modulation source. Respectively Internal Sinewave, associated input channel or opposite output channel.
            % 
            % :type depth: float 0-1, 0-125MHz or 0 - 360 deg
            % :param depth: Modulation depth (depends on modulation type): Fractional modulation depth, Frequency Deviation/Volt or Phase shift
            % 
            % :type frequency: float
            % :param frequency: Frequency of internally-generated sine wave modulation. This parameter is ignored if the source is set to ADC or DAC.
            % 
            % :raises ValueOutOfRangeException: if the channel number is invalid or modulation parameters can't be achieved
            params = struct('ch',ch,'mtype',mtype,'source',source,'depth',depth);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'gen_modulate', params);
        end

        function gen_modulate_off(obj, varargin)
            % GEN_MODULATE_OFF()
            % GEN_MODULATE_OFF(...,param1,val1,param2,val2,...)
            % Turn off modulation for the specified output channel.
            % 
            % If *ch* is None (the default), both channels will be turned off,
            % otherwise just the one specified by the argument.
            % 
            % :type ch: int; {1,2} or None
            % :param ch: Output channel to turn modulation off.
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'gen_modulate_off', params);
        end

        function gen_off(obj, varargin)
            % GEN_OFF()
            % GEN_OFF(...,param1,val1,param2,val2,...)
            % Turn Waveform Generator output(s) off.
            % 
            % The channel will be turned on when configuring the waveform type but can be turned off
            % using this function. If *ch* is None (the default), both channels will be turned off,
            % otherwise just the one specified by the argument.
            % 
            % :type ch: int; {1,2} or None
            % :param ch: Channel to turn off, or both.
            % 
            % :raises ValueError: invalid channel number
            % :raises ValueOutOfRangeException: if the channel number is invalid
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'gen_off', params);
        end

        function gen_rampwave(obj, ch, amplitude, frequency, varargin)
            % GEN_RAMPWAVE(ch, amplitude, frequency)
            % GEN_RAMPWAVE(...,param1,val1,param2,val2,...)
            % Generate a Ramp with the given parameters on the given channel.
            % 
            % This is a wrapper around the Square Wave generator, using the *riserate* and *fallrate*
            % parameters to form the ramp.
            % 
            % :type ch: int; {1,2}
            % :param ch: Channel on which to generate the wave
            % 
            % :type amplitude: float, volts
            % :param amplitude: Waveform peak-to-peak amplitude
            % 
            % :type frequency: float, hertz
            % :param frequency: Frequency of the wave
            % 
            % :type offset: float, volts
            % :param offset: DC offset applied to the waveform
            % 
            % :type symmetry: float, 0-1
            % :param symmetry: Fraction of the cycle rising.
            % 
            % :type phase: float, degrees 0-360
            % :param phase: Phase offset of the wave
            % 
            % :raises ValueError: invalid channel number
            % :raises ValueOutOfRangeException: invalid waveform parameters
            params = struct('ch',ch,'amplitude',amplitude,'frequency',frequency);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'gen_rampwave', params);
        end

        function gen_sinewave(obj, ch, amplitude, frequency, varargin)
            % GEN_SINEWAVE(ch, amplitude, frequency)
            % GEN_SINEWAVE(...,param1,val1,param2,val2,...)
            % Generate a Sine Wave with the given parameters on the given channel.
            % 
            % :type ch: int; {1,2}
            % :param ch: Channel on which to generate the wave
            % 
            % :type amplitude: float, [0.0,2.0] Vpp
            % :param amplitude: Waveform peak-to-peak amplitude
            % 
            % :type frequency: float, [0,250e6] Hz
            % :param frequency: Frequency of the wave
            % 
            % :type offset: float, [-1.0,1.0] Volts
            % :param offset: DC offset applied to the waveform
            % 
            % :type phase: float, [0-360] degrees
            % :param phase: Phase offset of the wave
            % 
            % :raises ValueError: if the channel number is invalid
            % :raises ValueOutOfRangeException: if wave parameters are out of range
            params = struct('ch',ch,'amplitude',amplitude,'frequency',frequency);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'gen_sinewave', params);
        end

        function gen_squarewave(obj, ch, amplitude, frequency, varargin)
            % GEN_SQUAREWAVE(ch, amplitude, frequency)
            % GEN_SQUAREWAVE(...,param1,val1,param2,val2,...)
            % Generate a Square Wave with given parameters on the given channel.
            % 
            % :type ch: int; {1,2}
            % :param ch: Channel on which to generate the wave
            % 
            % :type amplitude: float, volts
            % :param amplitude: Waveform peak-to-peak amplitude
            % 
            % :type frequency: float, hertz
            % :param frequency: Frequency of the wave
            % 
            % :type offset: float, volts
            % :param offset: DC offset applied to the waveform
            % 
            % :type duty: float, 0-1
            % :param duty: Fractional duty cycle
            % 
            % :type risetime: float, 0-1
            % :param risetime: Fraction of a cycle taken for the waveform to rise
            % 
            % :type falltime: float 0-1
            % :param falltime: Fraction of a cycle taken for the waveform to fall
            % 
            % :type phase: float, degrees 0-360
            % :param phase: Phase offset of the wave
            % 
            % :raises ValueError: invalid channel number
            % :raises ValueOutOfRangeException: input parameters out of range or incompatible with one another
            params = struct('ch',ch,'amplitude',amplitude,'frequency',frequency);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'gen_squarewave', params);
        end

        function gen_trigger_off(obj, varargin)
            % GEN_TRIGGER_OFF()
            % GEN_TRIGGER_OFF(...,param1,val1,param2,val2,...)
            % Turn off trigger/sweep mode for the specified output channel.
            % 
            % If *ch* is None (the default), both channels will be turned off,
            % otherwise just the one specified by the argument.
            % 
            % :type ch: int; {1,2} or None
            % :param ch: Output channel to turn trigger/sweep mode off
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'gen_trigger_off', params);
        end

        function set_defaults(obj)
            % SET_DEFAULTS()
            % Can be extended in implementations to set initial state 
            params = struct();
            mokuctl(obj, 'set_defaults', params);
        end

        function set_trigger(obj, ch, mode, varargin)
            % SET_TRIGGER(ch, mode)
            % SET_TRIGGER(...,param1,val1,param2,val2,...)
            % Configure gated, start, ncycle or sweep trigger mode on target channel.
            % 
            % The trigger event can come from an ADC input channel, the opposite generated waveform, the external
            % trigger input (for hardware that supports that) or a internally-generated clock of configurable
            % period.
            % 
            % The trigger event can be used in several different ways:
            % - *gated*: The output waveform is only generated while the trigger is asserted
            % - *start*: The output waveform is enabled once the trigger event fires
            % - *ncycle*: The output waveform starts at a trigger event and completes the given number of cycles, before turning off and re-arming
            % - *sweep*: The trigger event starts the waveform generation at the *sweep_start_freq*, before automatically sweeping the
            % frequency to *sweep_end_freq* over the course of *sweep_duration* seconds.
            % 
            % :type ch : int
            % :param ch: target channel.
            % 
            % :type mode: string, {'gated', 'start', 'ncycle', 'sweep', 'off'}
            % :param mode: Select the mode in which the trigger is operated.
            % 
            % :type ncycles : int, [1, 1e6]
            % :param ncycles : integer number of signal repetitions in ncycle mode.
            % 
            % :type sweep_start_freq : float, [0.0,250.0e6], hertz
            % :param sweep_start_freq : starting sweep frequency, set to current waveform frequency if not specified. Value range may vary for different waveforms.
            % 
            % :type sweep_end_freq : float, [0.0,250.0e6], hertz
            % :param sweep_end_freq : finishing sweep frequency. Value range may vary for different waveforms.
            % 
            % :type sweep_duration : float, [0.0,1000.0], seconds
            % :param sweep_duration : sweep duration in seconds.
            % 
            % :type trigger_source: string {'external', 'in', 'out', 'internal'}
            % :param: defines which source should be used as triggering signal.
            % 
            % :type trigger_threshold: float, [-5, 5], volts
            % :param trigger_threshold: The threshold value range dependes on the source and the attenution used. Values ranges might be less for different settings.
            % 
            % :type internal_trig_period: float, [0,1e11], seconds
            % :param internal_trig_period: period of the internal trigger clock, if used.
            % 
            % :type internal_trig_high: float, [0,1e11], seconds
            % :param internal_trig_high: High time of the internal trigger clock, if used. Must be less than the internal trigger period.
            params = struct('ch',ch,'mode',mode);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_trigger', params);
        end

    end
end
